% Lecture 5
%  SVM on checkers
% 
% S. Canu, feb 2014, scanu@insa-rouen.fr

% sample size
n = 500; % upto n = 10000; 

randn('seed',2);
rand('seed',2);
sigma=1.4; 
[Xapp,yapp,Xtest,ytest]=dataset_KM('checkers',n,n^2,sigma);
[n,p] = size(Xapp);

figure(1); 
clf; 
set(gcf,'Color',[1,1,1])
hold on
h1=plot(Xapp(yapp==1,1),Xapp(yapp==1,2),'+r'); 
set(h1,'LineWidth',2);
h2=plot(Xapp(yapp==-1,1),Xapp(yapp==-1,2),'db'); 
set(h2,'LineWidth',2);

%% with CVX

C = 10000;

kernel = 'gaussian';
kerneloption = .5;
lambda = eps^.5;

K=svmkernel(Xapp,kernel,kerneloption);
G = (yapp*yapp').*K;
e = ones(n,1);

fprintf('          CVX: ')
tic
cvx_begin
   cvx_precision best
   cvx_quiet(true)
   variable a(n)  
   dual variables de dp dC
   minimize( 1/2*a'*G*a - e'*a )
   subject to
      de : yapp'*a == 0;
      dp :  a >= 0;
      dC :  a <= C;
cvx_end
toc

%% with Mon QP

fprintf('      MonQP: ')
tic
[alpha,b,pos] = monqp(G,e,yapp,0,C,lambda,0);
toc

%% plot on a grid

[xtest1 xtest2]  = meshgrid([-1:.01:1]*3,[-1:0.01:1]*3); 
nn = length(xtest1); 
Xgrid = [reshape(xtest1 ,nn*nn,1) reshape(xtest2 ,nn*nn,1)]; 
%--------------  Evaluating the decision function
Kgrid = svmkernel(Xgrid,kernel,kerneloption,Xapp(pos,:));
ypred = Kgrid*(yapp(pos).*alpha) + b;

ypred = reshape(ypred,nn,nn); 

figure(1); 
clf; 
%contourf(xtest1,xtest2,ypred,50);shading flat;
hold on
contourf(xtest1,xtest2,ypred,50); shading flat; 
[cc,hh]=contour(xtest1,xtest2,ypred,[-1  0 1],'k');
clabel(cc,hh); 
set(hh,'LineWidth',2);
h1=plot(Xapp(yapp==1,1),Xapp(yapp==1,2),'+r'); 
set(h1,'LineWidth',2);

h2=plot(Xapp(yapp==-1,1),Xapp(yapp==-1,2),'db'); 
set(h2,'LineWidth',2);
xsup = Xapp(pos,:);
h3=plot(xsup(:,1),xsup(:,2),'ok'); 
set(h3,'LineWidth',2);
set(gca,'FontSize',14,'FontName','Times','XTick',[ ],'YTick',[ ],'Box','on');  

axis([-3 3 -3 3]); 
